from django.db import models
from django.utils.translation import gettext_lazy as _
from artd_partner.models import Partner
from artd_product.models import Product


class StockBase(models.Model):
    """Model definition for Stock."""

    created_at = models.DateTimeField(
        _("Created at"),
        help_text=_("Created at"),
        auto_now_add=True,
    )
    updated_at = models.DateTimeField(
        _("Updated at"),
        help_text=_("Updated at"),
        auto_now=True,
    )
    status = models.BooleanField(
        _("Status"),
        default=True,
        help_text=_("Status"),
    )
    partner = models.ForeignKey(
        Partner,
        on_delete=models.CASCADE,
        help_text=_("Partner"),
    )
    product = models.ForeignKey(
        Product,
        on_delete=models.CASCADE,
        help_text=_("Product"),
    )
    stock = models.IntegerField(
        _("Stock"),
        help_text=_("Stock"),
        default=0,
    )
    stock_min = models.IntegerField(
        _("Stock min"),
        help_text=_("Stock min"),
        default=0,
    )
    stock_max = models.IntegerField(
        _("Stock max"),
        help_text=_("Stock max"),
        default=0,
    )

    class Meta:
        abstract = True


class StockLog(StockBase):
    """Model definition for Stock."""

    class Meta:
        verbose_name = "Stock log"
        verbose_name_plural = "Stock logs"

    def __str__(self):
        """Unicode representation of Stock."""
        return str(self.id)


class Stock(StockBase):
    class Meta:
        unique_together = ("partner", "product")
        verbose_name = _("Stock list")
        verbose_name_plural = _("Stock lists")
        ordering = ("-created_at",)

    def __str__(self):
        return str(self.id)

    def save(self, *args, **kwargs):
        StockLog.objects.create(
            partner=self.partner,
            product=self.product,
            stock=self.stock,
            stock_min=self.stock_min,
            stock_max=self.stock_max,
        )
        super(Stock, self).save(*args, **kwargs)
